function vglobal = gist(img, Nori, Nscales, Nsubsampled_rows, Nsubsampled_cols, mask)
% function [vglobal] = gist(img, Nori, Nscales, Nsubsampled_rows,Nsubsampled_cols)
%
% vglobal is computed by averaging non-overlapping blocks of the magnitude of the local features.
% vglobal is a column vector.
% Vglobal can be further reduced by PCA to act as the gist.
%
% we use Eero Simoncelli's steerable pyramid toolbox.
% vglobal has size vglobal_rows x vglobal_cols x nfilters x ncolors
% where nfilters = nscales*norientations.

% code by Antonio Toralba and Aude Oliva, heavily modified by James Hays.
% mask is optional.  If it exists, it will be multiplied by each steerable
% pyramid band.  It should be the resolution of the full image.

srows = Nsubsampled_rows;
scols = Nsubsampled_cols;

% For 128x128 maximum Nsc=4, Nori=6
% For 64x64 maximum Nsc=3, Nori=4
switch Nori
    case 4, pyrFilters = 'sp3Filters'; 
    case 6, pyrFilters = 'sp5Filters';
    otherwise, 
        error(sprintf('unknown Nori %d', Nori))
end
edges='reflect1'; % repeat

nfilters = Nscales * Nori;
nbands = Nscales * Nori + 2;
% we use all bands except the first (too fine grained) and last (residual low pass)
vglobal = zeros(srows, scols, nbands-2);

[spyr.features, spyr.ind] = buildSpyr(img, Nscales, pyrFilters, edges);

% assert(nbands == size(spyr.ind, 1));
% Extract subbands and compute average features in subwindows:

for b=2:nbands-1

    x =  abs(pyrBand(spyr.features, spyr.ind, b));

    if(exist('mask'))
        
        %the bands get smaller and smaller, this will keep downsampling the
        %mask as necessary.
        if(size(x) ~= size(mask))
            mask = imresize(mask, [size(x,1) size(x,2)], 'bilinear');
        end
        
        x = x .* mask;
%         figure(2)
%         imagesc(mask)
%         truesize
    end
    
%     figure(1)
%     imagesc(x)
%     truesize
%     pause(.1)
    
%     imwrite(x, strcat(num2str(b),'steerable_pyr_img.bmp'));
    
%     figure(b)
%     imagesc(x)

    
%     x =  pyrBand(spyr.features, spyr.ind, b);
%     figure(1)
%     imagesc(x)
%     truesize
%     pause
%     size(x)
%     mean(mean(x))

    vglobal(:,:,b-1) =  down_sample(x, srows, scols);

    %removed multiplication by number of pixels
    %we can re-weight the different bands later.
%     vglobal(:,:,b-1) = vglobal(:,:,b-1) * size(x,1);
    
end

% vglobal = vglobal(:);
% lets not vectorize it in case we want to downsample it later