% calc_photo_priors_jh - Given an RGB input image (assumed to be in the
%		range 0-1), calculates per-channel joint histograms of pixel 
%		intensities occurring in four-connected neighborhoods near the 
%		center of the image as in:
%
%		"Priors for Large Photo Collections and What They Reveal about Cameras," 
%		S. Kuthirummal, A. Agarwala, D. B Goldman, and S. K. Nayar, European
%		Conference on Computer Vision, pp.74-87, Oct, 2008.
%
function [photo_priors_jh] = calc_photo_priors_jh(input_image);
	HALF_SIZE = 15;

	% Offset 0-255 => 1-256 for MATLAB array addressing
	input_image = im2uint8(input_image) + 1;	
	
	photo_priors_jh(:,:,1) = calc_joint_histogram(input_image(:,:,1), HALF_SIZE);
	photo_priors_jh(:,:,2) = calc_joint_histogram(input_image(:,:,2), HALF_SIZE);
	photo_priors_jh(:,:,3) = calc_joint_histogram(input_image(:,:,3), HALF_SIZE);
end

function [JH] = calc_joint_histogram(img, HALF_SIZE);
	center_x = floor(size(img,2) / 2);
	center_y = floor(size(img,1) / 2);

	JH = zeros(256);
	for x = center_x-HALF_SIZE:center_x+HALF_SIZE
		for y = center_y-HALF_SIZE:center_y+HALF_SIZE
			intensity = img(y,x);
			neighbors = [img(y,x-1) img(y-1,x) img(y,x+1) img(y+1,x)];
				
			for i = 1:4
				JH(intensity, neighbors(i)) = JH(intensity, neighbors(i)) + 1;
			end
		end
	end
end
