#!/usr/bin/python

#Image querying script written by Tamara Berg,
#and extended heavily by James Hays,
#and modified by Frank Palermo.

#9/26/2007 added dynamic timeslices to query more efficiently.
#8/18/2008 added new fields and set maximum time slice.
#8/19/2008 this is a much simpler function which gets ALL geotagged photos of
# sufficient accuracy.  No queries, no negative constraints.
# divides up the query results into multiple files
# 1/5/2009
# now uses date_taken instead of date_upload to get more diverse blocks of images

import sys, string, math, time, socket
from flickrapi2 import FlickrAPI
from datetime import datetime

socket.setdefaulttimeout(30)  #30 second time out on sockets before they throw
#an exception.  I've been having trouble with urllib.urlopen hanging in the 
#flickr API.  This will show up as exceptions.IOError.

#the time out needs to be pretty long, it seems, because the flickr servers can be slow
#to respond to our big searches.

###########################################################################
# Modify this section to reflect your data and specific search 
###########################################################################
# flickr auth information:
# change these to your flickr api keys and secret
flickrAPIKey = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"  # API key
flickrSecret = "YYYYYYYYYYYYYYYY"                  # shared "secret"

# change this to the location where you want to put your output file
out_path = "/tmp/";

# Set 'mintime' and 'endtime' to the Unix timestamps corresponding to the
#	endpoints of your desired query. It is recommended that you query for
#	no more than ten years of images with each invocation of the script,
#	and that you avoid querying for images which lie near the boundaries
#	between decades. Example values of 'mintime' and 'endtime' are provided
#	for the 1930s through the 1970s.
#mintime = -1262196000	#01/02/1930
#endtime = -946922400	#12/30/1939
#mintime = -946663200	#01/02/1940
#endtime = -631303200	#12/30/1949
mintime = -631044000 	#01/02/1950
endtime = -315770400 	#12/30/1959
#mintime = -315511200 	#01/02/1960
#endtime = -151200		#12/30/1969
#mintime =  108000 		#01/02/1970
#endtime =  315381600 	#12/30/1979
###########################################################################
# make a new FlickrAPI instance
fapi = FlickrAPI(flickrAPIKey, flickrSecret)

out_file = open(out_path + string.replace(str(mintime),'-','N') + '.txt','w')
total_images_queried = 0;

# number of seconds to skip per query  
#timeskip = 62899200 #two years
timeskip = 604800  #one week
#timeskip = 172800  #two days
#timeskip = 86400 #one day
#timeskip = 3600 #one hour
#timeskip = 2257 #for resuming previous query

maxtime = mintime+timeskip

#this is the desired number of photos in each block
desired_photos = 600

print datetime.fromtimestamp(mintime)
print datetime.fromtimestamp(endtime)

# prints out meta data to a file. An example is in 'eiffel tower.txt'
# where you have the data organized as follows:
# photo: id secret server
# title: title string
# tags: tags separated by spaces

while (maxtime < endtime):

    #new approach - adjust maxtime until we get the desired number of images 
    #within a block. We'll need to keep upper bounds and lower
    #lower bound is well defined (mintime), but upper bound is not. We can't 
    #search all the way from endtime.
    
    lower_bound = mintime + 900 #lower bound OF the upper time limit. must be at least 15 minutes or zero results
    upper_bound = mintime + timeskip * 20 #upper bound of the upper time limit
    maxtime     = math.floor(.950 * lower_bound + .050 * upper_bound)
    
    print '\nBinary search on time range upper bound' 
    print 'Lower bound is ' + str(datetime.fromtimestamp(lower_bound))
    print 'Upper bound is ' + str(datetime.fromtimestamp(upper_bound))
    
    keep_going = 15 #search stops after a fixed number of iterations
    while( keep_going > 0):
        
        try:
            rsp = fapi.photos_search(api_key=flickrAPIKey,
                                    media="photos",
                                    content_type="1",
                                    per_page="250", 
                                    page="1",
                                    min_taken_date=str(datetime.fromtimestamp(mintime)),
                                    max_taken_date=str(datetime.fromtimestamp(maxtime)))
            #we want to catch these failures somehow and keep going.
            time.sleep(1)
            fapi.testFailure(rsp)
            total_images = rsp.photos[0]['total'];
            null_test = int(total_images); #want to make sure this won't crash later on for some reason
            null_test = float(total_images);
        
            print '\nnumimgs: ' + total_images
            print 'mintime: ' + str(mintime) + ' maxtime: ' + str(maxtime) + ' timeskip:  ' + str(maxtime - mintime)
            
            if( int(total_images) > desired_photos ):
                print 'too many photos in block, reducing maxtime'
                upper_bound = maxtime
                maxtime = math.floor((lower_bound + maxtime) / 2) #midpoint between current value and lower bound.
                
            if( int(total_images) < desired_photos):
                print 'too few photos in block, increasing maxtime'
                lower_bound = maxtime
                maxtime = math.floor((upper_bound + maxtime) / 2)
                
            print 'Lower bound is ' + str(datetime.fromtimestamp(lower_bound))
            print 'Upper bound is ' + str(datetime.fromtimestamp(upper_bound))
            
            if( int(total_images) > 0): #only if we're not in a degenerate case
                keep_going = keep_going - 1
            
        except KeyboardInterrupt:
            print('Keyboard exception while querying for images, exiting\n')
            raise
        except Exception, inst:
            print sys.exc_info()[0]
            #print type(inst)     # the exception instance
            #print inst.args      # arguments stored in .args
            #print inst           # __str__ allows args to printed directly
            print ('Exception encountered while querying for images\n')

    #end of while binary search    
    print 'finished binary search'
        
    s = '\nmintime: ' + str(mintime) + ' maxtime: ' + str(maxtime)
    print s
    out_file.write(s + '\n') 

    i = getattr(rsp,'photos',None)
    if i:
                
        s = 'numimgs: ' + total_images
        print s
        out_file.write(s + '\n')
            
        current_image_num = 1;
            
        num = int(rsp.photos[0]['pages'])
        s =  'total pages: ' + str(num)
        print s
        out_file.write(s + '\n')
            
        #only visit 16 pages max, to try and avoid the dreaded duplicate bug
        #16 pages = 4000 images, should be duplicate safe.  Most interesting pictures will be taken.
            
        num_visit_pages = min(16,num)
            
        s = 'visiting only ' + str(num_visit_pages) + ' pages ( up to ' + str(num_visit_pages * 250) + ' images)'
        print s
        out_file.write(s + '\n')
            
        total_images_queried = total_images_queried + min((num_visit_pages * 250), int(total_images))

        #print 'stopping before page ' + str(int(math.ceil(num/3) + 1)) + '\n'
        
        pagenum = 1;
        while( pagenum <= num_visit_pages ):
        #for pagenum in range(1, num_visit_pages + 1):  #page one is searched twice
            print '  page number ' + str(pagenum)
            try:
                rsp = fapi.photos_search(api_key=flickrAPIKey,
                                        media="photos",
                                        content_type="1",
                                        per_page="250", 
                                        page=str(pagenum),
                                        sort="interestingness-desc",
                                        extras = "tags, original_format, license, geo, date_taken, date_upload, o_dims, views",
                                        min_taken_date=str(datetime.fromtimestamp(mintime)),
                                        max_taken_date=str(datetime.fromtimestamp(maxtime)))
                time.sleep(1)
                fapi.testFailure(rsp)
            except KeyboardInterrupt:
                print('Keyboard exception while querying for images, exiting\n')
                raise
            except:
                print sys.exc_info()[0]
                #print type(inst)     # the exception instance
                #print inst.args      # arguments stored in .args
                #print inst           # __str__ allows args to printed directly
                print ('Exception encountered while querying for images\n')
            else:

                # and print them
                k = getattr(rsp,'photos',None)
                if k:
                    m = getattr(rsp.photos[0],'photo',None)
                    if m:
                        for b in rsp.photos[0].photo:
                            if b!=None:
                                out_file.write('photo: ' + b['id'] + ' ' + b['secret'] + ' ' + b['server'] + '\n')
                                out_file.write('owner: ' + b['owner'] + '\n') 
                                out_file.write('title: ' + b['title'].encode("ascii","replace") + '\n')
                                
                                out_file.write('originalsecret: ' + b['originalsecret'] + '\n')
                                out_file.write('originalformat: ' + b['originalformat'] + '\n')
                                out_file.write('o_height: ' + b['o_height'] + '\n')
                                out_file.write('o_width: ' + b['o_width'] + '\n')
                                out_file.write('datetaken: ' + b['datetaken'].encode("ascii","replace") + '\n')
                                out_file.write('dateupload: ' + b['dateupload'].encode("ascii","replace") + '\n')
                                
                                out_file.write('tags: ' + b['tags'].encode("ascii","replace") + '\n')
                                
                                out_file.write('license: ' + b['license'].encode("ascii","replace") + '\n')
                                out_file.write('latitude: '  + b['latitude'].encode("ascii","replace") + '\n')
                                out_file.write('longitude: ' + b['longitude'].encode("ascii","replace") + '\n')
                                out_file.write('accuracy: '  + b['accuracy'].encode("ascii","replace") + '\n')
                                
                                out_file.write('views: ' + b['views'] + '\n')
                                out_file.write('interestingness: ' + str(current_image_num) + ' out of ' + str(total_images) + '\n');
                                out_file.write('\n')
                                current_image_num = current_image_num + 1;
                pagenum = pagenum + 1;  #this is in the else exception block.  It won't increment for a failure.

        #this block is indented such that it will only run if there are no exceptions
        #in the original query.  That means if there are exceptions, mintime won't be incremented
        #and it will try again
        timeskip = maxtime - mintime #used for initializing next binary search
        mintime  = maxtime
        
        if(total_images_queried > 20000):
            out_file.write('Total images queried: ' + str(total_images_queried) + '\n')
            out_file.close
        
            out_file = open(out_path + string.replace(str(mintime),'-','N') + '.txt','w')
            total_images_queried = 0;
