% calc_cluster_jhs - Given cluster data structures CF and CS (from 
%	initialize_clusters or split_clusters), extracts the RGB intensity
%	co-occurrence joint histograms (see calc_photo_priors_jh) from each image
%	descriptor and sums them to return cumulative histograms for each cluster.
%	The cumulative cluster histograms are returned in a structure array 
%	jh_struct where jh_struct(i) contains the cumulative histogram for cluster
%	i, and struct members JHR, JHG, and JHB contain arrays of doubles 
%	representing the histograms of each respective RGB color channel.
%	
function [jh_struct] = calc_cluster_jhs(CF, CS);
	% -- Settings --
	BINS = 256;
	% -- End Settings --
	numClusters = numel(CS);

	jh_struct.JHR = zeros(BINS);
	jh_struct.JHG = zeros(BINS);
	jh_struct.JHB = zeros(BINS);
		
	for i = 1:numClusters
		jh.JHR = zeros(BINS);
		jh.JHG = zeros(BINS);
		jh.JHB = zeros(BINS);
		
		for i2 = 1:CS(i)
			desc = load(CF{i,i2});

			jh.JHR = jh.JHR + desc.photo_priors_jh(:,:,1);
			jh.JHG = jh.JHG + desc.photo_priors_jh(:,:,2);
			jh.JHB = jh.JHB + desc.photo_priors_jh(:,:,3);
		end
		
		jh_struct(i) = jh;
	end
end
