function [interestingness, latt, long, date_vec, title, tags, ...
          photographer, license, accuracy] = parse_metadata2(Comment)
% takes a jpg comment field from imfinfo and parses the cell array of
% strings for some of the values. This code could be somewhat slow if 
% you only want a single one of these pieces of metadata.
%
% blah.Comment
% 
% photo: 3059572473 0073371b0c 3187
% owner: 42682565@N00
% title: ??
% originalsecret: null
% originalformat: null
% o_height: null
% o_width: null
% datetaken: 2008-08-14 22:09:25
% dateupload: 1227665360
% tags: city sunset sea summer cloud mountain canada beach bike vancouver
% license: 0
% latitude: 49.281468
% longitude: -123.17871
% accuracy: 10
% views: 35
% interestingness: 3 out of 239
% url: http:// ...
%
% interestingness is a 1x2 vector containing the rank and total images, such
% as interestingness 5 out of 492.
%
% date_vec is a 1x6 array of year, month, day, hour, minute, second
% title and tags are strings
% photographer is a scalar (the @N00 being removed from each name)
% license is a scalar.
% accuracy is a scalar.

if(~iscell(Comment))
    fprintf('!!parse_metadata:  Error, metadata should be in cell array!!\n');
    return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[t, r] = strtok(Comment{2});  %photographer:
if( ~strcmp(t, 'owner:') )
    fprintf('!!Error, the owner metadata for this image is not at the expected indices\n')
else
%     fprintf('full photographer string is %s\n', r);
    photographer = str2double(r(1:end-4)); 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[t, r] = strtok(Comment{3});  %title:
if( ~strcmp(t, 'title:') )
    fprintf('!!Error, the title metadata for this image is not at the expected indices\n')
else
    title = r; 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[t, r] = strtok(Comment{8});  %datetaken:
if( ~strcmp(t, 'datetaken:') )
    fprintf('!!Error, the date metadata for this image is not at the expected indices\n')
else
    % date_number = datenum(r, 'yyyy-mm-dd HH:MM:SS');
    % date_str = r; 
    date_vec = datevec(r, 'yyyy-mm-dd HH:MM:SS'); %divides into 1x6 array of year, month, day, hour, minute, second
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[t, r] = strtok(Comment{10});  %tags:
if( ~strcmp(t, 'tags:') )
    fprintf('!!Error, the tag metadata for this image is not at the expected indices\n')
else
    tags = r; 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[t, r] = strtok(Comment{11});  %license:
if( ~strcmp(t, 'license:') )
    fprintf('!!Error, the license metadata for this image is not at the expected indices\n')
else
    license = str2double(r); 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[t, r] = strtok(Comment{12});  %latitude:
if( ~strcmp(t, 'latitude:') )
    fprintf('!!Error, the gps metadata for this image is not at the expected indices\n')
else
    [t, r] = strtok(r); % number1
    latt = str2double(t);

    [t, r] = strtok(Comment{13});  %longitude: 
    [t, r] = strtok(r); % number1
    long = str2double(t);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[t, r] = strtok(Comment{14});  %accuracy:
if( ~strcmp(t, 'accuracy:') )
    fprintf('!!Error, the accuracy metadata for this image is not at the expected indices\n')
else
    accuracy = str2double(r); 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
interestingness = [0 0];

[t, r] = strtok(Comment{16});  %interestingness:
if( ~strcmp(t, 'interestingness:') )
    fprintf('!!Error, the interestingness metadata for this image is not at the expected indices\n')
else
    [t, r] = strtok(r); % number1
    interestingness(1,1) = str2double(t);
    [t, r] = strtok(r); % out
    [t, r] = strtok(r); % of
    [t, r] = strtok(r); % number2
    interestingness(1,2) = str2double(t);
end
